/*****************************************************************************
 *
 * Author: Steve Dunn, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef CLASSIFICATION_HPP
#define CLASSIFICATION_HPP


//class NodeType;

class Classification {
    protected:
        int numElements;
        char *contextLabel;   //e.g. "Location"
        char **labels;        //e.g. {"home","office","restaurant"...}
        char *postUNL;        //e.g. "/classifications/location"
        double *probability;

    public:
        Classification();
        Classification(char *contextLabel, char* postUNL);
        virtual ~Classification();

        const static double MIN_PROBABILITY = 1e-10;

        virtual int getNumElements();
        virtual char* getLabel(int index);
        virtual int getIndex(char *label);
        virtual double getProbability(int index);
        virtual double getProbability(char *label);
        virtual void getProbabilities(double* buffer);
                
        virtual void setProbability(int index, double prob);
        virtual void setElement(int index, char *label, double prob);
        virtual void appendElement(char *label, double prob);
        virtual void addToLabel(char *label, double prob);
        
        virtual void zero();
        virtual void empty();
        virtual void normalize();

        virtual char* mostLikely();
        virtual int mostLikelyIndex();
        virtual int meetsThreshold(char *label, double threshold);

        virtual void show();
        //virtual void publish(Context *context, char *unl);
        virtual void publish();
        virtual void publish(char *fullUNL);
        virtual void publishVector(char *postUNL);
        //virtual void load(NodeType *node);
};

#endif

