/*****************************************************************************
*
* Author: Wen Dong, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2005 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************
* This is an example on how to use the influence model with continuous 
* observations. Includes reference code for Matlab
*****************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include "hd_influence.h"
//#include <dmalloc.h>

// gcc -o psc -g -lgsl -lgslblas hd_influence.c psc.c

int power_station_c(){
  double ns[] = {2,2,2,2,2,2};
  double nox[] = {1,1,1,1,1,1};
  double A[] = {.99,.01,.08,.92};
  double Bx[] = {1,2};
  double D[] = {.33,.33,.33,0,0,0, .33,.33,.33,0,0,0, .33,.33,.33,0,0,0,
                0,0,0,.33,.33,.33, 0,0,0,.33,.33,.33, 0,0,0,.33,.33,.33};
  double prior[] = {.5,.5,.5,.5,.5,.5,.5,.5,.5,.5,.5,.5};
  double o_typex[] = {'c','c','c','c','c','c'};
  double cov[] = {.5f};
  int nc = 6;
  int i,j;
  int T = 5000;
  hd_sample *samplx;
  hd_sample_view *sampl_viewx;
  hd_influence * inflx;
  FILE * fd1;

  inflx = hd_influence_init(nc, ns, nox,o_typex);
  hd_influence_set_prior(inflx,prior);
  for(i=0;i<nc;i++){
    for(j=0;j<nc;j++){
      hd_influence_set_A(inflx,i,j,A);
    }
  }
  for(i=0;i<nc;i++){
    hd_influence_set_B(inflx,i,Bx);
    for(j=0;j<ns[i];j++)
      hd_influence_set_cov(inflx,i,j,cov);
  }
  hd_influence_set_D(inflx,D);
  hd_influence_set_H(inflx);
  fprintf(stdout,"########## the influence model is set as #######\n");
  hd_influence_print(stdout, inflx);

  samplx = hd_sample_alloc(inflx,T);
  hd_influence_sample(inflx,samplx);
  fprintf(stdout,"########## we sample for 5 time slices #######\n");
  fd1 = fopen("psc.smpl","wt");
  hd_sample_print(fd1,samplx);
  fclose(fd1);

  sampl_viewx = hd_sample_view_alloc(inflx,10,5);
  for(i=0;i<40;i++){
    double tmp = i+5;
    hd_sample *sampl = hd_sample_view_shift(sampl_viewx,1);
    hd_sample_set_o(sampl,0,4,&tmp);
    fprintf(stdout,"@@@@@@@ %d @@@@@@@\n",i);
    hd_sample_print(stdout,sampl);
  }
  hd_sample_view_free(sampl_viewx);
  return 000;

  //for(i=0;i<10;i++){
  hd_influence_reset_ess(inflx);
  hd_influence_enter_evidence(inflx,samplx);
  fprintf(stdout,"loglik=%f\n",hd_influence_loglikelihood(inflx,samplx));  
  fprintf(stdout,"### the sample statistics are computed as follows ###\n");
  hd_sample_print_statistics(stdout,samplx);

  hd_influence_maximize_params(inflx);
  fprintf(stdout,"### learning result after one iteration ###\n");
  hd_influence_print(stdout, inflx);
  //}
  
  hd_sample_free(samplx);


  hd_influence_free(inflx);

  return 0;
}

int main (int argc, char** argv){
  power_station_c();
  return 0;
}


/************************************************************** 
 *  here is the matlab script to verify the result 
addpath 'the FullBNT path'
add_BNT_to_path
addpath 'the influence model path'
ev = textread('psc.smpl');
ev = num2cell(ev(:,7:12)');

bnet = mk_influence(2*ones(1,6), 1*ones(1,6), 'cccccc');
for i=1:bnet.nchains
  for j=1:bnet.nchains
    bnet.A{i,j} = [.99 .01;.08 .92];
  end
end
for i=1:bnet.nchains
  bnet.Pi{i} = [.5 .5];
end
for i=1:bnet.nchains
  bnet.B{i} = [1;2]; 
  bnet.cov{i} = repmat(.5*eye(bnet.m_outputs(i)),[1 1 bnet.n_states(1)]);
end
bnet.T = 0*bnet.T;
bnet.T(1:3,1:3) = 1/3*ones(3);
bnet.T(4:6,4:6) = 1/3*ones(3);
engine = influence_inf_engine(bnet);
[engine1,ll] = enter_evidence(bnet,ev);
% check engine1.obslik, engine1.alpha, engine1.beta, engine1.scale
engine1.obslik
engine1.alpha
engine1.beta
engine1.scale
bnet1 = maximize_params(engine1);
% check maximize_params
influence_matrix(bnet1)
bnet1.B{:}
bnet1.cov{:}
bnet1.Pi{:}
 ************************************************************/
