/*****************************************************************************
*
* Author: Wen Dong, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2005 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************
* This is an example on how to use the influence model with discrete 
* observations. Includes reference code for Matlab
*****************************************************************************/

#include <stdio.h>
#include "hd_influence.h"

// gcc -g -lgsl -lgslblas hd_influence.c psd.c -o psd

int power_station_d(){
  double ns[] = {2,2,2,2,2,2};
  double no[] = {2,2,2,2,2,2};
  double A[] = {.99,.01,.08,.92};
  double B[] = {.9,.1,.1,.9};
  double D[] = {.33,.33,.33,0,0,0, .33,.33,.33,0,0,0, .33,.33,.33,0,0,0,
                0,0,0,.33,.33,.33, 0,0,0,.33,.33,.33, 0,0,0,.33,.33,.33};
  double prior[] = {.5,.5,.5,.5,.5,.5,.5,.5,.5,.5,.5,.5};
  double o_type[] = {'d','d','d','d','d','d'};
  int nc = 6;
  int i,j;
  int T = 10;
  hd_sample *sampl;
  hd_influence * infl;
  FILE * fd1;

  infl = hd_influence_init(nc, ns, no,o_type);
  hd_influence_set_prior(infl,prior);
  for(i=0;i<nc;i++){
    for(j=0;j<nc;j++){
      hd_influence_set_A(infl,i,j,A);
    }
  }
  for(i=0;i<nc;i++){
    hd_influence_set_B(infl,i,B);
  }
  hd_influence_set_D(infl,D);
  hd_influence_set_H(infl);
  fprintf(stdout,"########## the influence model is set as #######\n");
  hd_influence_print(stdout,infl);

  sampl = hd_sample_alloc(infl,T);
  hd_influence_sample(infl,sampl);
  fprintf(stdout,"########## we sample for %d time slices #######\n",T);
  fd1 = fopen("psd.smpl","wt");
  hd_sample_print(fd1,sampl);
  fclose(fd1);

  hd_influence_reset_ess(infl);
  hd_influence_enter_evidence(infl,sampl);

  fprintf(stdout,"### the sample statistics are computed as follows ###\n");
  hd_sample_print_statistics(stdout,sampl);

  hd_influence_maximize_params(infl);
  fprintf(stdout,"### learning result after one iteration ###\n");
  hd_influence_print(stdout,infl);

  hd_sample_free(sampl);
  hd_influence_free(infl);
  return 0;
}

int main (int argc, char** argv){
  power_station_d();
  return 0;
}



/************************************************************** 
 *  here is the matlab script to verify the result 
addpath 'the FullBNT path'
add_BNT_to_path
addpath 'the influence model path'
ev = textread('psd.smpl');
ev = num2cell(ev(:,7:12)'+1);

bnet = mk_influence(2*ones(1,6), 2*ones(1,6), 'dddddd');
for i=1:bnet.nchains
  for j=1:bnet.nchains
    bnet.A{i,j} = [.99 .01;.08 .92];
  end
end
for i=1:bnet.nchains
  bnet.Pi{i} = [.5 .5];
end
for i=1:bnet.nchains
  bnet.B{i} = [.9 .1;.1 .9]; 
end
bnet.T = 0*bnet.T;
bnet.T(1:3,1:3) = 1/3*ones(3);
bnet.T(4:6,4:6) = 1/3*ones(3);
engine = influence_inf_engine(bnet);
[engine1,ll] = enter_evidence(engine,ev);
% check engine1.obslik, engine1.alpha, engine1.beta, engine1.scale
engine1.obslik
engine1.alpha
engine1.beta
engine1.scale
bnet1 = maximize_params(engine1);
% check maximize_params
influence_matrix(bnet1)
bnet1.B{:}
bnet1.cov{:}
bnet1.Pi{:}
 ************************************************************/





