/*************************************************
 * File: unl.h
 * Birth Date: Jul 29, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2002 MIT Media Lab,
 *               2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#ifndef ENCHANTMENT_UNL_H
#define ENCHANTMENT_UNL_H

/* Returns the server portion of the UNL. */
const char *ExtractUNLServer(const char *UNL);

/* Returns the path portion of the UNL. */
const char *ExtractUNLPath(const char *UNL);

/* Returns the last node name of the UNL. */
const char *ExtractUNLEndNode(const char *UNL);

/* Returns the parent of the UNL. */
const char *ExtractParentUNL(const char *UNL);

/* Returns the first node name of the path. */
const char *GetUNLHead(const char *path);

/* Returns the path after the first node name. */
const char *GetUNLTail(const char *path);

/* Creates a UNL with the given server and path. */ 
const char *CreateUNL(const char *server, const char *path);

/* Creates a UNL with the given path relative to the given base. */
const char *AppendUNLPath(const char *baseUNL, const char *appPath);

/* 
 * If des is a descendant of anc, returns its path relative to anc;
 * otherwise returns NULL. 
 */
const char *DescendsFrom(const char *anc, const char *des);

#endif
