/*************************************************
 * File: PureToneSignal.c
 * Birth Date: Nov 26, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *************************************************/

#include <math.h>
#include <enchantment/signals.h>
#include <enchantment/time-utils.h>
#include <unistd.h>
#include <stdlib.h>

double freq;
double sample;

void generateAccel(signed char *value, int i)
{
  value[0] = sqrt((cos(i * 2 * M_PI * freq * sample / 1000000) + 1) / 2) * 127;
  value[1] = 0;
  value[2] = 0;
}

void generateAudio(signed char *value, int i)
{
  value[0] = (cos(i * 2 * M_PI * freq * sample / 1000000)) * 127;
}

int main(int argc, char **argv)
{
  int i = 0;
  signed char value[3];
  struct timeval stamp;
  struct timeval now;
  char *type;
  Signal *signal;
  void (*generate)(signed char *value, int i);

  gettimeofday(&stamp, NULL);

  if (argc < 3)
    {
      printf("Usage: %s [type] [frequency]\n", argv[0]);
      exit(1);
    }

  signal = NewSignalGenUNL("tone");
  SignalSetType(signal, T_INT8);

  type = argv[1];
  if (!strcmp(type, "accel"))
    {
      sample = 20000;
      generate = generateAccel;
      SignalSetDimension(signal, 1, 3, NULL, NULL);
    }
  else if (!strcmp(type, "audio"))
    {
      sample = 20;
      generate = generateAudio;
      SignalSetDimension(signal, 1, 1, NULL, NULL);
    }
  else
    {
      fprintf(stderr, "Unknown type '%s'\n", type);
    }

  SignalSetSampleRate(signal, 1000000/sample, 1);
    
  freq = atof(argv[2]);
  printf("Tone at %g Hz\n", freq);

  while (1)
    {
      gettimeofday(&now, NULL);
      
      if (timeval_sub(&stamp, &now) > sample)
	usleep(sample);
      SignalGenAccept(signal);

      (*generate)(value, i);

      SignalSend(signal, &stamp, value);

      timeval_add(&stamp, sample);
      i++;
    }
}
