/*************************************************
 * File: TestSignal.c
 * Author: Daniel Barkalow
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 * 
 * Copyright (C) 2002 MIT Media Lab,
 *               2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *************************************************/

#include <enchantment/signals.h>

#include <stdlib.h>
#include <math.h>
#include <unistd.h>

int main(int argc, char **argv)
{
  Signal *signal;
  char *UNL = argv[1];
  char *format = "r412";
  char *buffer;
  int i;
  if (argc < 2 || argc > 3)
    {
      fprintf(stderr, "Usage: %s [UNL] {FORMAT}\n", argv[0]);
      exit(0);
    }
  if (argc > 2)
    format = argv[2];
  signal = NewSignalGenUNL(UNL);
  switch (format[1])
    {
    case '1':
      i = 0;
      break;
    case '2':
      i = 1;
      break;
    case '4':
      i = 2;
      break;
    case '8':
      i = 3;
      break;
    default:
      fprintf(stderr,
	      "Only sizes of 1, 2, 4, and 8 bytes/value are supported.\n");
      exit(1);
    }
  switch (format[0])
    {
    case 'r':
      SignalSetType(signal, T_NUMERIC + T_FLOAT_FORMAT + i);
      break;
    case 'i':
      SignalSetType(signal, T_NUMERIC + T_INT_FORMAT + i);
      break;
    default:
      fprintf(stderr, "Type codes: r=real, i=integer\n");
      exit(0);
    }
  //signal->eltsize = format[1] - '0';
  SignalSetDimension(signal, 1, 2, NULL, NULL);
  buffer = malloc(SignalBufferSize(signal));
  for (i = 0; i < 20; i++)
    {
      struct timeval stamp;
      sleep(1);
      SignalGenAccept(signal);
      if (SignalClientCount(signal))
	{
	  gettimeofday(&stamp, NULL);
	  switch (SignalGetType(signal))
	    {
	    case T_FLOAT:
	      ((float *)buffer)[0] = 2.0;
	      ((float *)buffer)[1] = exp(i);
	      break;
	    case T_DOUBLE:
	      // (network-byte-order double){7, 6, 5, 4, 3, 2, 1, 0}
	      ((double *)buffer)[0] = 5.447603722011605e-270;
	      ((double *)buffer)[1] = exp(i);
	      break;
	    case T_INT32:
	      ((int *)buffer)[0] = 2;
	      ((int *)buffer)[1] = i;
	      break;
	    default:
	      printf("Confused about type: %x\n", SignalGetType(signal));
	    }
	  SignalSend(signal, &stamp, buffer);
	}
    }
  SignalClose(signal);
  return 0;
}
