/*************************************************
 * File: dstring.c
 * Author: Jonathan Gips
 * Birth Date: 6/29/01
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 * 
 * Copyright (C) 2002 MIT Media Lab,
 *               2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#include <enchantment/dstring.h>

#include "hashtable.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/**** Constants ****/
#define NUM_STRING_BUCKETS  200
#define MULTIPLIER  -1664117991

static HashTable stringTable = NULL;

static char safetolower(char ch)
{
  return (isascii(ch) ? tolower(ch) : ch);
}

static int StringHashFn(const void *elem, int numBuckets)
{
  int i;
  unsigned long hashcode = 0;
  const char *s = *((char **) elem);
  
  for (i = 0; i < strlen(s); i++)
    hashcode = hashcode * MULTIPLIER + safetolower(s[i]);
  return (hashcode % numBuckets);
}

static int StringCompFn(const void *elemptr1, const void *elemptr2)
{
  char *s1 = *((char **) elemptr1);
  char *s2 = *((char **) elemptr2);
  return strcmp(s1, s2);
}

static void StringFreeFn(void *elem)
{
  free(*((char**)elem));
}

const char *InstanceNumber(int i)
{
  char buf[22];
  sprintf(buf, "%d", i);
  return InstanceString(buf);
}

const char *InstanceString(const char *string)
{
  char *dstring = NULL, **dptr = NULL;
  
  if (!string)
    return NULL;

  if (!stringTable)
    stringTable = TableNew(sizeof(char *), NUM_STRING_BUCKETS, 
			   StringHashFn, StringCompFn, StringFreeFn);
  
  dptr = ((char**) TableLookup(stringTable, &string));
  
  if (dptr)
    return *dptr;

  dstring = strdup(string);
      
  TableEnter(stringTable, &dstring);
  return dstring;
}

const char *InstanceStringN(const char *string, int len)
{
  char buff[len + 1];

  if (!string || len < 0) 
    return NULL;

  strncpy(buff, string, len);
  buff[len] = '\0';

  return InstanceString(buff);
}
