/*************************************************
 * File: optionset.c
 * Birth Date: Sep 15, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#include "optionset.h"

#include <stdlib.h>

typedef struct
{
  const char *key;
  char *value;
} OptionRec;

char *GetSetOption(DArray set, const char *key)
{
  int i;
  if (!set)
    return NULL;
  for (i = 0; i < ArrayLength(set); i++)
    {
      OptionRec *option = (OptionRec *) ArrayNth(set, i);
      if (option->key == key)
	return option->value;
    }
  return NULL;
}

static void freeOption(void *elem)
{
  OptionRec *rec = elem;
  free(rec->value);
}

DArray NewOptionSet()
{
  return ArrayNew(sizeof(OptionRec), 1, freeOption);
}

void AddSetOption(DArray set, const char *key, char *value)
{
  OptionRec option;
  option.key = key;
  option.value = value;
  ArrayAppend(set, &option);
}

void FreeOptionSet(DArray set)
{
  ArrayFree(set);
}
