/*************************************************
 * File: whiteboard-protocol.h
 * Birth Date: Jul 28, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2002 MIT Media Lab,
 *               2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#ifndef ENCHANTMENT_WHITEBOARD_PROTOCOL
#define ENCHANTMENT_WHITEBOARD_PROTOCOL

#define DEFAULT_PORT 11111

#define S_RETURN			"sr" // Return from a request

#define S_UPDATE			"su" // Update from a subscription

#define S_TERMINAL			"st" // Server going down

// Authenticated to have the given certificate
#define CERTIFICATE_AUTH "certificate:"

// Authenticated to have the given network
// (identifies "localhost" and a private network if one is configured)
#define NETWORK_AUTH "network:"

// Authenticated to have the given informational mark
// (These are at the client's discretion)
#define INFO_AUTH "info:"

// Authenticated to have the given client name
// (Chosen by the server, for a single connection)
#define NAME_AUTH "name:"

// use to act without authorization
#define  K_DROP_AUTH			"no-auth"

#define C_PUBLISH			"cp"
#define  K_PUBLISH_HARD			 "hard"
#define  K_PUBLISH_OPERATION		 "op"
#define   KV_PUBLISH_OPERATION_MERGE	  "m"
#define   KV_PUBLISH_OPERATION_ADDONLY	  "a"

#define C_LOCK				"cl"
#define  K_LOCK_TYPE			 "locktype"
#define   KV_LOCK_TYPE_NORMAL		  'n'
#define   KV_LOCK_TYPE_RECURSIVE	  'r'
#define   KV_LOCK_TYPE_REMOVE		  'R'
#define  K_LOCK_UPDATE			"allowupdate"

#define C_GETLOCK			"cgl"
#define  K_GETLOCK_OWNER		 "owner"
// Also  K_LOCK_TYPE

#define C_SUBSCRIBE			"cs"
#define  K_SUBSCRIBE_SELF		 "self"

#define C_UNSUBSCRIBE			"cS"

#define C_GET				"cg"

#define C_GET_META			"cgm"

#define C_SET_META			"cpm"

#define MAX_NAME_LEN 80

const char *NicknameUNL(const char *nickname);

/* Error codes from whiteboard.h */
/*
#define ENCHANT_SUCCESS		0x000 // Done
#define ENCHANT_NO_EFFECT	0x001 // Already like that
#define ENCHANT_SENT		0x002 // An update was sent
#define ENCHANT_NOT_EXIST	0x100 // Doesn't exist
#define ENCHANT_LOCKED		0x200 // Permissions problem
#define ENCHANT_TYPE		0x201 // Bad type
#define ENCHANT_UNSUPPORTED	0x300 // Couldn't do it
#define ENCHANT_ERROR		0x400 // Something wrong with request
#define ENCHANT_SYMLINK		0x500 // Need to use symlink
*/

#endif
