/*************************************************
 * File: introduction.h
 * Birth Date: 4/23/04
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 * 
 * Copyright (C) 2003 MIT Media Lab
 *               2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#ifndef INTRODUCTION_H
#define INTRODUCTION_H

#include <enchantment/types.h>
#include "socket.h"

typedef struct
{
  char *key;
  unsigned hasIndex : 1;
  TypeID expected;
} IntroFormat;

typedef struct 
{
  IntroFormat *format;
  int index;
  DataType value;
} IntroMessage;

/* Reads an intro message from the socket, looking it up in the table
 * of expected message formats. Returns -1 to indicate the end of the
 * set of messages, or the index of the matching line in tha table.
 */
int IntroRecv(Socket *socket, IntroFormat **table, IntroMessage *message);

/* Frees memory allocated for this message. */
void IntroFree(IntroMessage *message);

/* Sends an intro message to the socket. */
int IntroSend(Socket *socket, IntroMessage *message);

int IntroDone(Socket *socket);

#endif
