/*************************************************
 * File: tensor.c
 * Birth Date: Aug 18, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#include <enchantment/matrix.h>
#include <enchantment/vector.h>

#include <stdio.h>
#include <stdlib.h>

Vector *NewVector(unsigned length)
{
  Vector *ret = malloc(sizeof(Vector));
  ret->values = malloc(length * sizeof(double));
  ret->dimension = length;
  return ret;
}

Vector *CopyVector(Vector *v)
{
  Vector *ret = NewVector(v->dimension);
  memcpy(ret->values, v->values, ret->dimension * sizeof(double));
  return ret;
}

Matrix *NewMatrix(unsigned rows, unsigned cols)
{
  Matrix *ret = malloc(sizeof(Matrix));
  ret->rows = rows;
  ret->cols = cols;
  ret->values = malloc(rows * cols * sizeof(double));
  return ret;
}

Matrix *CopyMatrix(Matrix *m)
{
  Matrix *ret = NewMatrix(m->rows, m->cols);
  memcpy(ret->values, m->values, ret->rows * ret->cols * sizeof(double));
  return ret;
}

static int readValue(FILE *fp, double *ret_p)
{
  return fscanf(fp, " %lg", ret_p);
}

Vector *ReadVector(FILE *fp, unsigned length)
{
  Vector *ret = NewVector(length);
  int i;
  for (i = 0; i < length; i++)
    {
      double value;
      readValue(fp, &value);
      VectorSet(ret, i, value);
    }
  while (fgetc(fp) != '\n')
    ;
  return ret;
}

Matrix *ReadMatrix(FILE *fp, unsigned rows, unsigned cols)
{
  Matrix *ret = NewMatrix(rows, cols);
  int i, j;
  for (i = 0; i < rows; i++)
    {
      for (j = 0; j < cols; j++)
	{
	  double value;
	  readValue(fp, &value);
	  MatrixSet(ret, i, j, value);
	}
    }
  while (fgetc(fp) != '\n')
    ;
  return ret;
}

Vector *VectorSubtract(Vector *a, Vector *b)
{
  Vector *ret;
  int i;
  if (a->dimension != b->dimension)
    return NULL;
  ret = NewVector(a->dimension);
  for (i = 0; i < a->dimension; i++)
    {
      VectorSet(ret, i, VectorRef(a, i) - VectorRef(b, i));
    }
  return ret;
}

void PrintVector(FILE *fp, Vector *v)
{
  int i;
  fprintf(fp, "[");
  for (i = 0; i < v->dimension; i++) 
    {
      fprintf(fp, i ? " %g" : "%g",
	      VectorRef(v, i));
    }
  fprintf(fp, "]\n");
}
