/*************************************************
 * File: time-utils.c
 * Birth Date: Aug 23, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2003 MIT Media Lab,
 *               2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#include <enchantment/time-utils.h>
#include <stdlib.h>
#include <time.h>

void timeval_add(struct timeval *tm, long usecs)
{
  tm->tv_usec += usecs;
  if (usecs < 0)
    {
      while (tm->tv_usec < 0)
        {
          tm->tv_sec--;
          tm->tv_usec += 1000000;
        }
    }
  else
    {
      while (tm->tv_usec > 1000000)
        {
          tm->tv_sec++;
          tm->tv_usec -= 1000000;
        }
    }
}

long timeval_sub(struct timeval *t1, struct timeval *t2)
{
  return (t1->tv_sec - t2->tv_sec) * 1000000 +
    (t1->tv_usec - t2->tv_usec);
}

static long minsleep = 1000;
static long spintime = 1000;

void wait_until(struct timeval *tm, struct timeval *actual)
{
  struct timeval timestamp;
  struct timespec timer;
  long diff;
  if (!actual)
    actual = &timestamp;
  gettimeofday(actual, NULL);
  diff = timeval_sub(tm, actual);
  if (diff > 0)
    {
      while (diff > minsleep)
	{
	  timer.tv_sec = (diff - spintime) / 1000000;
	  timer.tv_nsec = ((diff - spintime) % 1000000) * 1000;
	  nanosleep(&timer, NULL);
	  gettimeofday(actual, NULL);
	  diff = timeval_sub(tm, actual);
	}
      while (diff > 0)
	{
	  gettimeofday(actual, NULL);
	  diff = timeval_sub(tm, actual);
	}
    }
  if (actual)
    *actual = timestamp;
}
